import {
  scaleLinear,
  extent,
  axisLeft,
  axisBottom,
  select,
  symbol,
  symbolCircle,
  symbolSquare,
  symbolStar,
  scaleOrdinal,
  schemeCategory10,
} from 'd3';

export const scatterPlot = () => {
  let width;
  let height;
  let data;
  let xValue;
  let yValue;
  let margin = { top: 10, right: 10, bottom: 50, left: 50 }; // Adjusted margins for axis labels
  let symbolType;
  let color = scaleOrdinal(schemeCategory10);

  const chart = (selection) => {
    selection.each(function () {
      // Create scales
      const xScale = scaleLinear()
        .domain(extent(data, xValue))
        .range([0, width - margin.left - margin.right]);
      const yScale = scaleLinear()
        .domain(extent(data, yValue))
        .range([height - margin.top - margin.bottom, 0]);

      // Create the root container
      const svg = select(this)
        .selectAll('svg')
        .data([null]);
      const svgEnter = svg.enter().append('svg');
      svgEnter
        .merge(svg)
        .attr('width', width)
        .attr('height', height);

      // Append a 'g' element for chart
      const gEnter = svgEnter.append('g');
      const g = svg.select('g').merge(gEnter);
      g.attr(
        'transform',
        `translate(${margin.left},${margin.top})`,
      );

      // X-axis and the x-axis label
      const xAxis = axisBottom(xScale).ticks(12);
      const xAxisGEnter = gEnter
        .append('g')
        .attr('class', 'x-axis')
        .attr(
          'transform',
          `translate(0,${height - margin.top - margin.bottom})`,
        );
      xAxisGEnter
        .call(xAxis)
        .append('text')
        .attr('class', 'axis-label')
        .attr('y', 70) // Adjust this to place the label below the x-axis line
        .attr('x', (width - margin.left - margin.right) / 2) // Center the label
        .attr('fill', 'red')
        .text('Years'); // Text for the x-axis label

      // Call the x-axis on update
      g.select('.x-axis').transition().call(xAxis);

      // Y-axis
      const yAxis = axisLeft(yScale).ticks(12);
      const yAxisGEnter = gEnter
        .append('g')
        .attr('class', 'y-axis');
      yAxisGEnter
        .call(yAxis)
        .append('text')
        .attr('class', 'axis-label')
        .attr('transform', 'rotate(-90)')
        .attr('y', -90) // Adjust this to move the label to the left of the y-axis line
        .attr(
          'x',
          -(height - margin.top - margin.bottom) / 2,
        ) // Center the label
        .attr('text-anchor', 'middle')
        .attr('fill', 'red')
        .text('Number of Death'); // Text for the y-axis label

      // Call the y-axis on update
      g.select('.y-axis').transition().call(yAxis);

      // Draw symbols
      const symbols = g.selectAll('.symbol').data(data);

      symbols
        .enter()
        .append('path')
        .attr('class', 'symbol')
        .merge(symbols)
        .attr(
          'd',
          symbol()
            .size(64)
            .type(symbolType ? symbolType : symbolCircle),
        )
        .attr(
          'transform',
          (d) =>
            `translate(${xScale(xValue(d))},${yScale(yValue(d))})`,
        )
        .attr('fill', (d) => color(d.type));
      symbols.exit().remove();
    });
  };

  chart.color = (_) =>
    arguments.length ? ((color = _), chart) : color;
  chart.width = (_) =>
    arguments.length ? ((width = +_), chart) : width;
  chart.height = (_) =>
    arguments.length ? ((height = +_), chart) : height;
  chart.data = (_) =>
    arguments.length ? ((data = _), chart) : data;
  chart.xValue = (_) =>
    arguments.length ? ((xValue = _), chart) : xValue;
  chart.yValue = (_) =>
    arguments.length ? ((yValue = _), chart) : yValue;
  chart.margin = (_) =>
    arguments.length ? ((margin = _), chart) : margin;
  chart.radius = (_) =>
    arguments.length ? ((radius = +_), chart) : radius;
  chart.symbolType = (_) =>
    arguments.length
      ? ((symbolType = _), chart)
      : symbolType;

  return chart;
};
